import LoreWarning from './lore.warning';
import TemplatesManager from './templates.manager';
export default class TemplateEditor {
    constructor(template, loreParserInstance) {
        this.template = template;
        this.loreParserInstance = loreParserInstance;
    }
    /*
     *
     * Set/Get Methods
     *
     */
    set(key, value) {
        const keyValue = this.template.keys[key];
        const val = value.toString();
        const separatedTemplates = TemplatesManager.getSeperatedTemplates(this.loreParserInstance.lore);
        const indexs = separatedTemplates.length === 1 ? [0] : TemplatesManager.getTemplateIndex(this.template, separatedTemplates);
        let lore = separatedTemplates;
        if (!this.template.isComplexTemplate) {
            for (const index of indexs) {
                if (val.length + lore[index].length > LoreWarning.MAX_LORE_LINE_LENGTH)
                    return new LoreWarning('MAX_LORE_LINE_LENGTH', index, val.length);
                let splitLore = lore[index].map((line) => line.split(TemplatesManager.MARKER));
                splitLore = splitLore.map((line, i) => this.replaceKeyByValue(keyValue, val, line, i));
                lore[index] = splitLore.map((line) => line.join(''));
                // WARNINGS MANAGER
                if (lore[index].some((line) => line.length >= LoreWarning.MAX_LORE_LINE_LENGTH))
                    return new LoreWarning('MAX_LORE_LINE_LENGTH', index, lore[index].filter((line) => line.length >= LoreWarning.MAX_LORE_LINE_LENGTH)[0].length);
            }
        }
        else {
            let splitLore = separatedTemplates.flat().map((line) => line.split(TemplatesManager.MARKER));
            splitLore = splitLore.map((line, i) => this.replaceKeyByValue(keyValue, val, line, i));
            lore = splitLore.map((line) => line.join(''));
        }
        this.loreParserInstance.lore = lore.flat();
    }
    get(key) {
        const keyValue = this.template.keys[key];
        const lineIndex = this.template.shape.findIndex((v) => v.includes(keyValue));
        if (this.loreParserInstance.lore.length <= lineIndex)
            return null;
        const targetLine = this.loreParserInstance.lore[lineIndex];
        const keyIndex = this.template.shape[lineIndex]?.split(TemplatesManager.MARKER)?.indexOf(keyValue);
        const value = targetLine.split(TemplatesManager.MARKER);
        return value[keyIndex] || null;
    }
    /*
     *
     * Util Method
     *
     */
    replaceKeyByValue(keyValue, value, line, i) {
        const keyIndexs = this.findIndexsOf(keyValue, this.template.shape[i].split(TemplatesManager.MARKER));
        if (keyIndexs.length > 0) {
            for (const idx of keyIndexs) {
                line[idx] = `${TemplatesManager.MARKER}${value}${TemplatesManager.MARKER}`;
            }
        }
        else {
            for (const key of Object.values(this.template.keys)) {
                const keyIndexs = this.findIndexsOf(key, this.template.shape[i].split(TemplatesManager.MARKER));
                for (const idx of keyIndexs) {
                    line[idx] = `${TemplatesManager.MARKER}${line[idx].replaceAll(TemplatesManager.MARKER, '')}${TemplatesManager.MARKER}`;
                }
            }
        }
        return line;
    }
    findIndexsOf(target, array) {
        return array.reduce((r, v, i) => r.concat(v === target ? i : []), []);
    }
}
