import { Player, world, system, EnchantmentTypes, GameMode, } from '@minecraft/server';
import Timer from 'utils/date.timer';
import MeleeWeapon from 'melee/melee-weapon';
import { LoadingBar } from 'utils/loading-bar';
// @ts-ignore
export default class CustomPlayer extends Player {
    constructor() {
        super(...arguments);
        this.holdingWeaponSlot = null;
        this.isHoldingWeapon = false;
        this.isAbleToUse = false;
        this.weapon = null;
        this.drawWeaponLastSlot = null;
        this.startedHoldingRightClickDate = null;
        this.startedHoldingRightClickIntervalId = null;
        this.bypassClickLeftHitDate = false;
        this.bypassClickRightHitDate = false;
        this.isInit = false;
    }
    init() {
        this.ableToUseDate = new Timer();
        this.ableToLeftClickHitDate = new Timer();
        this.ableToRightClickHitDate = new Timer();
        this.ableToHitDate = new Timer();
        this.isInit = true;
    }
    isAlive() {
        const health = this.getComponent('health');
        return health.currentValue > 0;
    }
    getInventory() {
        const inventoryComponent = this.getComponent('inventory');
        return inventoryComponent.container;
    }
    getHandItem() {
        return this.getInventory().getItem(this.selectedSlotIndex);
    }
    damageHandItem(damages = 1, bypassCreative = false) {
        if (!bypassCreative && this.getGameMode() === GameMode.creative)
            return;
        const itemStack = this.getHandItem();
        const durabilityComponent = itemStack.getComponent('durability');
        const enchantementsComponent = itemStack.getComponent('enchantable');
        const damageChance = durabilityComponent.getDamageChance(enchantementsComponent?.getEnchantment(EnchantmentTypes.get('unbreaking'))?.level || 0);
        const random = Math.random() * 100;
        if (random < damageChance) {
            durabilityComponent.damage += damages;
        }
        system.runTimeout(() => this.getInventory().setItem(this.selectedSlotIndex, itemStack), 2);
    }
    getScore(objectiveName) {
        if (!world.scoreboard.getObjective(objectiveName)?.hasParticipant(this))
            return 0;
        return world.scoreboard.getObjective(objectiveName)?.getScore(this) || 0;
    }
    resetScore(objectiveName) {
        if (!world.scoreboard.getObjective(objectiveName)?.hasParticipant(this))
            return 0;
        world.scoreboard.getObjective(objectiveName)?.removeParticipant(this);
    }
    setScore(objectiveName, score) {
        world.scoreboard.getObjective(objectiveName)?.setScore(this, score);
    }
    resetRightClickHolding() {
        if (this.startedHoldingRightClickIntervalId) {
            system.clearRun(this.startedHoldingRightClickIntervalId);
            this.startedHoldingRightClickIntervalId = null;
            this.startedHoldingRightClickDate = null;
        }
    }
    updateActionBar() {
        if (!this.weapon)
            return;
        if (!this.isAbleToUse && this.weapon.displayDrawCooldown) {
            this.clearedActionBar = false;
            const loadingBar = new LoadingBar(MeleeWeapon.LOADING_BAR_TIME_PER_SQUARE, '█', {
                done: '§a',
                default: '§f',
            }, this.weapon.drawTime, this.ableToUseDate.ending, Date.now()).create();
            this.onScreenDisplay.setActionBar(loadingBar);
        }
        else if (!this.ableToHitDate.isFinished && this.lastAttack && this.lastAttack.enabled && this.lastAttack.displayCooldown) {
            this.clearedActionBar = false;
            const loadingBar = new LoadingBar(MeleeWeapon.LOADING_BAR_TIME_PER_SQUARE, '█', {
                done: '§a',
                default: '§f',
            }, (this.lastAttack.cooldown * 1000) / 20, this.ableToHitDate.ending, Date.now()).create();
            this.onScreenDisplay.setActionBar(loadingBar);
        }
        else if (!this.clearedActionBar) {
            this.clearedActionBar = true;
            this.onScreenDisplay.setActionBar(' ');
        }
    }
}
for (const key of Object.getOwnPropertyNames(CustomPlayer.prototype)) {
    Player.prototype[key] = CustomPlayer.prototype[key];
}
world.afterEvents.playerSpawn.subscribe((data) => {
    const player = data.player;
    player.init();
});
for (const player of world.getAllPlayers()) {
    const customPlayer = player;
    customPlayer.init();
}
