import { world } from '@minecraft/server';
class DataBase {
    constructor(name, value, eventsMethods = {
        afterSave: () => { },
        afterLoad: () => { },
        beforeLoad: () => { },
        beforeSave: () => { },
    }) {
        this.name = name;
        this.eventsMethods = eventsMethods;
        this.objective = world.scoreboard.getObjective(this.name);
        this.data = null;
        this.data = value;
        if (!this.exist())
            this.create();
        else
            this.load();
    }
    /*
    
    Database main methodes
    
    */
    save() {
        if (!this.data)
            return;
        this.reset();
        if (this.eventsMethods.beforeSave)
            this.eventsMethods.beforeSave(this);
        let stringedData = JSON.stringify(this.data);
        if (stringedData.length >= DataBase.MAX_CHAR) {
            stringedData = this.chunckString(stringedData, DataBase.MAX_CHAR);
            stringedData.forEach((str, i) => this.objective.setScore(str, i));
        }
        else
            this.objective.setScore(stringedData, 0);
        if (this.eventsMethods.afterSave)
            this.eventsMethods.afterSave(this);
    }
    load() {
        if (this.eventsMethods.beforeLoad)
            this.eventsMethods.beforeLoad(this);
        if (this.objective?.getParticipants()?.length < 0)
            return;
        const participants = this.objective.getParticipants();
        const data = [];
        const maxScore = Math.max(...participants.map((participant) => this.objective.getScore(participant)));
        let oldScore = 0;
        this.objective.getParticipants().forEach((participant) => {
            const score = this.objective.getScore(participant);
            if (score === 0 || score - oldScore === 1 || score === maxScore) {
                data[score] = participant.displayName;
            }
            else {
                const diff = score - oldScore;
                for (let i = 0; i < diff; i++) {
                    data[score - i] = participant.displayName;
                }
            }
            oldScore = score;
        });
        if (data.length > 0)
            this.data = JSON.parse(data.join(''));
        if (this.eventsMethods.afterLoad)
            this.eventsMethods.afterLoad(this);
    }
    reset() {
        if (this.objective?.getParticipants()?.length > 0)
            this.objective.getParticipants().map((participant) => this.objective.removeParticipant(participant));
    }
    exist() {
        return !!world.scoreboard.getObjective(this.name);
    }
    create() {
        this.objective = world.scoreboard.addObjective(this.name, this.name);
    }
    /*
        
        Database formating methods
        
    */
    chunckString(str, x = DataBase.MAX_CHAR) {
        const chunks = [];
        for (let i = 0; i < str.length; i += x) {
            const chunk = str.slice(i, i + x);
            chunks.push(chunk);
        }
        return chunks;
    }
}
DataBase.MAX_CHAR = 32768 - 1;
export default DataBase;
