import MeleeWeapon, { IAttack, ICharge, IKnockback } from 'melee/melee-weapon';
import { IOtherParticles, IParticleParams } from 'melee/particles/particles';
import { IOtherSounds, ISoundsParams } from 'melee/sounds/sounds';

export default class Test extends MeleeWeapon {
	public itemTypeId: string = 'minecraft:stick';

	// MAIN MELEE PROPERTIES
	public name: string = 'Test Knife';

	public drawTime: number = 1050; // in ms
	public displayDrawCooldown: boolean = true;

	public leftAttack: IAttack = {
		enabled: true,
		damages: 23,
		delay: 0,
		cooldown: 100,
		displayCooldown: true,
		range: 5,
		precision: 0.07,
		knockback: {
			Ximpusle: 0.7,
			Zimpusle: 0.3,
		},
		sounds: {
			onHitBlock: {
				soundIds: ['csgo.knife.hitwall'],
				pitch: 1,
				volume: 1.5,
			},
			onHitEntity: {
				soundIds: ['csgo.knife.slash'],
				pitch: 1,
				volume: 1.5,
			},
			onMiss: {
				soundIds: ['mob.creeper.say'],
				pitch: 0.5,
				volume: 100,
			},
		},
		particles: {
			onHitBlock: {
				particleIds: ['move:tiger', 'move:tiger'],
			},
			onHitEntity: {
				particleIds: ['ag:blood', 'knife:sweep'],
			},
		},
	};

	public rightAttack: IAttack = {
		enabled: true,
		damages: 48,
		delay: 0,
		cooldown: 20,
		range: 3.5,
		precision: 0.07,
		knockback: {
			Ximpusle: 0.7,
			Zimpusle: 0.3,
		},
		sounds: {
			onHitBlock: {
				soundIds: ['csgo.knife.hitwall'],
				pitch: 1,
				volume: 1.5,
			},
			onHitEntity: {
				soundIds: ['csgo.knife.stab'],
				pitch: 1,
				volume: 1.5,
			},
			onMiss: {
				soundIds: ['mob.creeper.say'],
				pitch: 0.5,
				volume: 100,
			},
		},
		particles: {
			onHitBlock: {
				particleIds: ['move:tiger', 'hit:bullet5', 'move:tiger', 'move:tiger', 'move:tiger'],
			},
			onHitEntity: {
				particleIds: ['ag:blood', 'ag:blood', 'ag:blood', 'knife:sweep'],
			},
		},
	};

	// ATTACK on draw
	public initialAttack: IAttack = {
		enabled: false,
	};

	public sounds: Record<keyof IOtherSounds, ISoundsParams> = {
		onDraw: {
			soundIds: [''],
			pitch: 1,
			volume: 0.2,
		},
	};

	public particles: Record<keyof IOtherParticles, IParticleParams> = {
		onDraw: {
			particleIds: [''],
		},
	};

	// Charge System
	public charges: ICharge = {
		enabled: false,
	};

	// KNOCKBACK
	public knockback: IKnockback = {
		Ximpusle: 0.7,
		Zimpusle: 0.3,
	};
}
