import { PlayerSoundOptions, Vector3, Dimension } from '@minecraft/server';
import CustomPlayer from 'player/custom-player';

export interface ISoundsParams extends PlayerSoundOptions {
	soundIds?: Array<string>;
}

export interface IAttackSounds {
	onHitBlock?: ISoundsParams;
	onHitEntity?: ISoundsParams;
	onMiss?: ISoundsParams;
}

export interface IOtherSounds {
	onDraw?: ISoundsParams;
}

export default class Sounds {
	public static readonly DEFAULT_VOLUME: number = 1;
	public volumeMultiplier: number = Sounds.DEFAULT_VOLUME;

	public static playForPlayer(sound: ISoundsParams, player: CustomPlayer): void {
		if (sound && sound?.soundIds?.length > 0) player.playSound(Sounds.getRandomSoundId(sound), sound);
	}
	public static play(sound: ISoundsParams, dimension: Dimension, location: Vector3): void {
		if (sound && sound?.soundIds?.length > 0) dimension.playSound(Sounds.getRandomSoundId(sound), location, sound);
	}

	public static getRandomSoundId<T extends ISoundsParams>(soundParams: T): string {
		return soundParams.soundIds[Math.floor(Math.random() * soundParams.soundIds.length)];
	}
}
