export class Vector3 {
	x: number;
	y: number;
	z: number;

	constructor(x: number, y: number, z: number) {
		this.x = x;
		this.y = y;
		this.z = z;
	}

	// Méthode pour ajouter un autre vecteur à ce vecteur
	add(other: Vector3): Vector3 {
		return new Vector3(this.x + other.x, this.y + other.y, this.z + other.z);
	}

	// Nouvelle méthode pour multiplier ce vecteur par un autre vecteur
	multiply(other: Vector3): Vector3 {
		return new Vector3(this.x * other.x, this.y * other.y, this.z * other.z);
	}

	// Méthode pour ajouter un autre vecteur à ce vecteur
	rotate(angle: number): Vector3 {
		// Rotation de 45 degrés dans le sens antihoraire autour de l'axe vertical (y)
		const angleRadians = angle * (Math.PI / 180); // Math.PI / 4; // 45 degrés en radians

		// Utilisation de la formule de rotation 3D autour de l'axe y
		const newX = (this.x * Math.cos(angleRadians) - this.z * Math.sin(angleRadians)) * 2;
		const newZ = (this.x * Math.sin(angleRadians) + this.z * Math.cos(angleRadians)) * 2;

		// Création et retour d'un nouveau vecteur3 avec les nouvelles coordonnées
		return new Vector3(newX, this.y, newZ);
	}

	rotateVectorAroundY(angle: number) {
		const angleRadians = angle * (Math.PI / 180); // Math.PI / 4; // 45 degrés en radians

		const cosTheta = Math.cos(angleRadians);
		const sinTheta = Math.sin(angleRadians);

		// Calculer les nouvelles composantes du vecteur après rotation autour de l'axe Y
		const newX = this.x * cosTheta + this.z * sinTheta;
		const newY = this.y;
		const newZ = -this.x * sinTheta + this.z * cosTheta;

		// Retourner le nouveau vecteur avec les composantes mises à jour
		return new Vector3(newX, newY, newZ);
	}

	// Méthode pour soustraire un autre vecteur de ce vecteur
	subtract(other: Vector3): Vector3 {
		return new Vector3(this.x - other.x, this.y - other.y, this.z - other.z);
	}

	thicken(width: number): Vector3 {
		// Crée un nouveau vecteur avec la largeur ajoutée aux composantes x et z
		return new Vector3(this.x, this.y + width, this.z + width);
	}

	increaseLength(newLength: number): Vector3 {
		const currentLength = this.magnitude();
		if (currentLength === 0) {
			return new Vector3(newLength, 0, 0); // Vecteur initial nul
		}
		const scaleFactor = newLength / currentLength;
		return this.multiplyScalar(scaleFactor);
	}

	// Méthode pour multiplier ce vecteur par un scalaire
	multiplyScalar(scalar: number): Vector3 {
		return new Vector3(this.x * scalar, this.y * scalar, this.z * scalar);
	}

	// Méthode pour calculer le produit scalaire avec un autre vecteur
	dot(other: Vector3): number {
		return this.x * other.x + this.y * other.y + this.z * other.z;
	}

	// Méthode pour calculer le produit vectoriel avec un autre vecteur
	cross(other: Vector3): Vector3 {
		const newX = this.y * other.z - this.z * other.y;
		const newY = this.z * other.x - this.x * other.z;
		const newZ = this.x * other.y - this.y * other.x;
		return new Vector3(newX, newY, newZ);
	}

	// Méthode pour calculer la norme (magnitude) du vecteur
	magnitude(): number {
		return Math.sqrt(this.x * this.x + this.y * this.y + this.z * this.z);
	}

	// Méthode pour normaliser le vecteur (le rendre unitaire)
	normalize(): Vector3 {
		const mag = this.magnitude();
		if (mag === 0) {
			return new Vector3(0, 0, 0); // Éviter la division par zéro
		}
		return new Vector3(this.x / mag, this.y / mag, this.z / mag);
	}

	// Méthode pour convertir le vecteur en une chaîne lisible
	toString(): string {
		return `(${this.x}, ${this.y}, ${this.z})`;
	}
}
